// Mesh Algorithms: a library of algorithms to manipulate 3D meshes
// Copyright (C) 2001 Tim Garthwaite and Jason Reposa

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef COLOR_H
#define COLOR_H

#include "options.h"
#include <cstdio>
#include <string>
#include <GL/gl.h>

//!  Holds color information for a Vertex.
class Color {
  GLfloat red, green, blue, alpha;
//  bool created;

 public:
  //Color(); //!< Default constructor.
  //! Constructor with given color attributes. default color is white fully opaque
  Color( GLfloat r = 1.0f, //!< red.
	 GLfloat g = 1.0f, //!< green,
	 GLfloat b = 1.0f, //!< blue.
	 GLfloat a = 1.0f //!< alpha. default value is 1.0.
       );

  //! Setter for color attributes.
  GLvoid setColor( GLfloat r, //!< red.
		   GLfloat g, //!< green.
		   GLfloat b, //!< blue.
		   GLfloat a = 1.0f //!< alpha.
                 );

  GLvoid r(GLfloat r); //!< setter for red.
  GLvoid g(GLfloat g); //!< setter for green.
  GLvoid b(GLfloat b); //!< setter for blue.
  GLvoid a(GLfloat a); //!< setter for alpha.

  GLfloat r() const; //!< Getter for red.
  GLfloat g() const; //!< Getter for green.
  GLfloat b() const; //!< Getter for blue.
  GLfloat a() const; //!< Getter for alpha.

  GLfloat & r(); //!< Reference to red.
  GLfloat & g(); //!< Reference to green.
  GLfloat & b(); //!< Reference to blue.
  GLfloat & a(); //!< Reference to alpha.
  std::string toString(); //!< Returns a string showing attributes.
//  bool exists(); //!< True if the color exists after decimation.

};

#endif
