// Mesh Algorithms: a library of algorithms to manipulate 3D meshes
// Copyright (C) 2001 Tim Garthwaite and Jason Reposa

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef DECIMATE_H
#define DECIMATE_H

#include <set>
#include "Scene.h"

//!  Decimate class. This class has a function 'remove' that does the decimation of the model.
class Decimate {
  /*!
    This is the set of triangles that are inserted when a vertex is removed
    see: removeVertex() and more specifically triangulate()
  */
  std::set <Triangle *> insert;

  /*!
    Helper function to check for concavities
  */
  bool can_create_link(Vertex * c_n, Vertex * m_n, Vertex * l_n, Vertex * v_t_r);
  bool check_overlap(std::vector <Vertex *> * verts, std::set <Triangle *> * tris);
  bool vertex_contained(Vertex * v, Triangle * t);
  bool ordered(Vertex * v0, Vertex * v1, Vertex * v2, bool clockwise);

  // we need the vertex that is going to be removed in triangulate
  Vertex * current_vertex_to_remove;

  bool removeVertex(Model * model, Vertex * vertex_to_remove); //!< Removes a vertex during decimation.
public:
  Decimate();
  bool remove(Scene * scene, int maxNumRemoved);
};

#endif
