// Mesh Algorithms: a library of algorithms to manipulate 3D meshes
// Copyright (C) 2001 Tim Garthwaite and Jason Reposa

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef POINT_H
#define POINT_H

#include "options.h"
#include <cstdio>
#include <string>
#include <GL/gl.h>

//! Class definition for Point
class Point {
  GLfloat p[4]; //!< Local storage space for the coordinates of the Point

 public:

  //! Constructor
  Point(GLfloat x = 0.0f, //!< The x coordinate
	GLfloat y = 0.0f, //!< The y coordinate
	GLfloat z = 0.0f, //!< The z coordinate
	GLfloat w = 1.0f); //!< The w coordinate (space) ... should always be 1.0, otherwise, should normalize the Point into the first space

  GLfloat x() const; //!< Getter for the x coordinate
  GLfloat y() const; //!< Getter for the y coordinate
  GLfloat z() const; //!< Getter for the z coordinate
  GLfloat w() const; //!< Getter for the w coordinate

  GLfloat &x(); //!< Reference to (setter for) the x coordinate
  GLfloat &y(); //!< Reference to (setter for) the y coordinate
  GLfloat &z(); //!< Reference to (setter for) the z coordinate
  GLfloat &w(); //!< Reference to (setter for) the w coordinate

  //! Set the coordinates of the Point
  GLvoid setPoints(GLfloat x, GLfloat y, GLfloat z, GLfloat w = 1.0f);

  //! Return coordinates of the Point in string form
  std::string toString();

};

#endif
