// Mesh Algorithms: a library of algorithms to manipulate 3D meshes
// Copyright (C) 2001 Tim Garthwaite and Jason Reposa

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef TRIANGLE_H
#define TRIANGLE_H

class Triangle;

#include "options.h"
#include <string>
#include "Vertex.h"
#include "Normal.h"
#include "Color.h"
#include "Point.h"
#include <GL/gl.h>

//! Triangle class definition
class Triangle {
  Vertex * v[3]; //!< The three Vertices that make up this Triangle
  Color color; //!< The Color of this Triangle
  Normal normal; //!< The Normal to this Triangle
  Point center; //!< The center Point of this Triangle
  bool replaced_before; //!< Whether or not the Triangle has been replaced before. If a vertex removal fails we need to know which triangles have been replaced so we can "un" replace them.

  //! Helper function to caluculate the Color of this Triangle
  GLvoid calculateColor(Vertex * v0, Vertex * v1, Vertex * v2);

  //! Helper function to calculate the Normal to this Triangle
  GLvoid calculateNormal(Vertex * v0, Vertex * v1, Vertex * v2);

  //! Helper function to calculate the center Point of this Triangle
  GLvoid calculateCenter(Vertex * v0, Vertex * v1, Vertex * v2);

 public:

  //! Constructor
  /*!
    The Vertices must be speified in clockwise order when looking at the
    front of the Triangle
  */
  Triangle(
    Vertex * v0, //!< The first Vertex
    Vertex * v1, //!< The second Vertex
    Vertex * v2  //!< The third Vertex
  );

  Vertex * v0(); //!< The first Vertex
  Vertex * v1(); //!< The second Vertex
  Vertex * v2(); //!< The third Vertex

  //! Set the Color of the Vertex
  GLvoid setColor(Color * c);

  //! Get the Color of the Vertex
  Color * getColor();

  //! Set the Normal to the Vertex explicitly
  GLvoid setNormal(Normal * n);
  Normal * getNormal();

  GLvoid setCenter(Point * c);
  Point * getCenter();

  bool replaced();
  bool replace(Vertex * v1, Vertex *v2);

  std::string toString();
  bool contains(Vertex * v1);

};

#endif
