// Mesh Algorithms: a library of algorithms to manipulate 3D meshes
// Copyright (C) 2001 Tim Garthwaite and Jason Reposa

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef VRML_PARSER_H
#define VRML_PARSER_H

#include <string>
#include <fstream>
#include "Scene.h"

//! Class definition for the VRMLParser, which parses and writes VRML files
class VRMLParser {
  std::ifstream in;
  std::ofstream out;
  std::vector <Vertex *> parseVertices();
  std::vector <GLint> parseCoordIndex();
  std::vector <Color *> parseColor();
  std::vector <Color *> setColor(Color * c);

  std::vector <Vertex *> vertices;
  std::vector <GLint> coordIndex;
  std::vector <Color *> colors;

  /*
    Helper variables that help parsing
  */
  GLvoid addModelToScene(Scene * s);
  Color * diffuseColor;
  Vector translate;
  bool clockwise;
  std::vector <Vertex *> vertexList;
  bool verticesFinished;
  bool coordIndexFinished;
  bool colorsFinished;
  bool diffuseColorFinished;
  bool normalsFinished;
  bool normalIndexFinished;
  bool foundTranslation;

  GLint numVertex;

  public:

    //! Constructor
    VRMLParser();

    //! Parse the contents of file <filename> into a Scene object
    /*!
      This is a very basic, single-pass linear algorithm which
      reads the contents of <filename> into lists of Vertices,
      Triangles, Normals, and Colors.  Along the way, it sets all
      this information into the corresponding objects and calculates
      extra information such as Triangle Colors, center Points, and
      Normals, which we use in the decimation algorithm.
    */
    Scene * parse(std::string filename);

    //! Write the contents of the loaded Scene to file <filename>
    /*!
      This is a very basic, heirarchically called algorithm which
      steps iteratively through all Models in the Scene, all Triangles
      in the Model, and all Vertices in the Triangles, using standard
      C++ output streams to write the coordinates into a VRML file.
    */
    bool write(Scene * scene, std::string filename);
};

#endif
