// Mesh Algorithms: a library of algorithms to manipulate 3D meshes
// Copyright (C) 2001 Tim Garthwaite and Jason Reposa

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <cmath>
#include "Vector.h"

using namespace std;

Vector::Vector (const float x, const float y, const float z, const float w) {
  data[0] = x;
  data[1] = y;
  data[2] = z;
  data[3] = w;
}

float Vector::x() const { 
  return(data[0]); 
}

float Vector::y() const { 
return(data[1]); 
}

float Vector::z() const { 
return(data[2]); 
}

float Vector::w() const { 
return(data[3]); 
}

float & Vector::x() { 
  return(data[0]); 
}

float & Vector::y() { 
  return(data[1]); 
}

float & Vector::z() { 
  return(data[2]); 
}

float & Vector::w() { 
  return(data[3]); 
}

float Vector::magnitude() const {

  // compute the inner product
  float a = dot(*this, *this);

  // compute the sqrt of the 
  return (float)sqrt(a);

}

void Vector::set(float nx, float ny, float nz) {
	data[0] = nx;
	data[1] = ny;
	data[2] = nz;
	data[3] = 1.0;
}

void Vector::normalize() {

  // compute the magnitude
  // btw magnitude does not compute correctly
  float length = (float)sqrt(data[0]*data[0] + data[1]*data[1] + data[2]*data[2]);

  if (length == 0.0f)
    length = 1.0f;

  // normalize
  data[0] /= length;
  data[1] /= length;
  data[2] /= length;
  data[3] = 1.0;
}
 
Vector operator + (const Vector &v1, const Vector &v2) {
  return Vector(v1.x() + v2.x(),
		v1.y() + v2.y(),
		v1.z() + v2.z());
}

Vector operator - (const Vector &v1, const Vector &v2) {
  return Vector(v1.x() - v2.x(),
		v1.y() - v2.y(),
		v1.z() - v2.z(),
		v1.w());
}

Vector operator * (const Vector &v1, const float t) {
  return Vector(v1.x() * t,
		v1.y() * t,
		v1.z() * t,
		v1.w());
}

float dot (const Vector &v1, const Vector &v2) {
  return (v1.x() * v2.x() + v1.y() * v2.y() + v1.z() * v2.z() + v1.w()*v2.w());
}

Vector cross (const Vector &v1, const Vector &v2) {
  return Vector(v1.y() * v2.z() - v1.z() * v2.y(),
		v1.z() * v2.x() - v1.x() * v2.z(),
		v1.x() * v2.y() - v1.y() * v2.x(),
		1.0);
}

string Vector::toString() {
  string s;
  char c[40];

  sprintf(c, "%f %f %f %f", x(), y(), z(), w());
  s = c;

  return s;
}

