// Mesh Algorithms: a library of algorithms to manipulate 3D meshes
// Copyright (C) 2001 Tim Garthwaite and Jason Reposa

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef VECTOR_H
#define VECTOR_H

#include <string>
#include <cstdio>

//! Vector class definition
class Vector {
  //! Local data
  float data[4];

 public:
  //! Constructor
  Vector( const float x=0.0, //!< x value of the vector.
          const float y=0.0, //!< y value of the vector.
          const float z=0.0, //!< z value of the vector.
          const float w=1.0 //!< Space the vector exists in.
        );

  // Access methods
  float   x () const; //!< Getter for the x value.
  float   y () const; //!< Getter for the y value.
  float   z () const; //!< Getter for the z velue.
  float   w () const; //!< Getter for the vector's space.

  float & x (); //!< Reference to the x value.
  float & y (); //!< Reference to the y value.
  float & z (); //!< Reference to the z value.
  float & w (); //!< Reference to the vector's space.

  // Methods
  float magnitude() const; //!< Returns the magnitude of the vector.
  void  normalize(); //!< Normalizes the vector (brings to the 1 space).
  void  set(float nx, float ny, float nz); //!< Set all values.

  // Operators
  friend Vector operator + (const Vector &v1, const Vector &v2); //!< Addition.
  friend Vector operator - (const Vector &v1, const Vector &v2); //!< Subtraction.
  friend float  dot        (const Vector &v1, const Vector &v2); //!< Dot product.
  friend Vector cross      (const Vector &v1, const Vector &v2); //!< Cross product.
  std::string toString(); //!< Returns a string showing the vector's values.

};

#endif

