// Mesh Algorithms: a library of algorithms to manipulate 3D meshes
// Copyright (C) 2001 Tim Garthwaite and Jason Reposa

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef VIEWPORT_CONTROLLER_H
#define VIEWPORT_CONTROLLER_H

#define VIEWPORT_CONTROLLER_DEBUG false

#include "options.h"
#include <vector>
#include <cmath>
#include "Viewport.h"

//! Class definition for the ViewportController
/*!
  This is the interface between GLUT callbacks and the Viewport object
*/
class ViewportController {
  std::vector <Viewport> viewportList;
  Viewport * currentViewport;
  int width;
  int height;
  std::string filename;
  Viewport * remove(Viewport * v);
  Viewport * getViewport(int viewportID);
  void fixViewportIDs(int viewportID);

 public:
  ViewportController(int maxNumViewports, int w, int h);
  ~ViewportController();

  void renderScene();
  int numViewports();
  Scene * getScene(int viewportID);
  bool selectCurrentViewport(int x, int y);
  bool deleteViewport();
  bool createViewport(std::string file = "NONE_SPECIFIED");
  bool resizeScene(int Width, int Height);

  std::string getFilename();
  int getWidth();
  int getHeight();

  bool save();
  bool save(std::string file);
  bool decimateEvent(GLfloat f, GLint TYPE);

  void reportStatistics();
  void resetViewport();
  void makeSmooth();
  void makeFlat();
  void switchViewPoints();
  void switchViewEdges();
  void switchViewTriangles();
  void switchLighting();
  void switchNormals();
  void switchVertNormals();

  void setCameraControl(int control);

  void decreaseX_Off();
  void increaseX_Off();
  void decreaseY_Off();
  void increaseY_Off();
  void decreaseZ_Off();
  void increaseZ_Off();

  void mouseX_Rot(float f);
  void mouseY_Rot(float f);
  void mouseZ_Rot(float f);

  void decreaseX_Rot();
  void increaseX_Rot();
  void decreaseY_Rot();
  void increaseY_Rot();
  void decreaseZ_Rot();
  void increaseZ_Rot();
};

#endif

