// Mesh Algorithms: a library of algorithms to manipulate 3D meshes
// Copyright (C) 2001 Tim Garthwaite and Jason Reposa

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
/* SET WINDOWS COMPILE OPTIONS */
#ifdef _WIN32
#define WIN32_LEAN_AND_MEAN
#define NOWINRES
#define NOSERVICE
#define NOMCX
#define NOIME
#include <windows.h>
#pragma comment (linker, "/ENTRY:mainCRTStartup")
#pragma comment (linker, "/SUBSYSTEM:CONSOLE")
#endif
/* END WINDOWS COMPILE OPTIONS */

#include "ViewportController.h"
#include <iostream>
#include <GL/glut.h>
#include <sstream>

#define ERR_CLEAN_EXIT 0x0001
#define ERR_PRINT_USAGE 0x0002
#define ERR_FILE_NOT_FOUND 0x003
#define ERR_CONTINUE 0x004
#define ERR_UNKNOWN 0x00F

#define PROGRAM_TITLE "Mesh Simplification - Decimation (WPI)"

#define DEC_20_PER 0x0101
#define DEC_40_PER 0x0102
#define DEC_60_PER 0x0103
#define DEC_80_PER 0x0104
#define DEC_90_PER 0x0105
#define DEC_ONE_TENTH 0x0106
#define DEC_ONE 0x0107
#define DEC_TWO_HALF 0x0108
#define DEC_FIVE 0x0109
#define DEC_TEN 0x010A

#define SHOW_LIGHTING 0x0201
#define SHOW_NORMALS 0x0202
#define SHOW_VERT_NORMALS 0x0203

#define SHOW_POINTS 0x0204
#define SHOW_EDGES 0x0205
#define SHOW_TRIANGLES 0x0206

#define ADD_VIEWPORT 0x0301
#define REMOVE_VIEWPORT 0x0302

#define MODEL_CONTROL 0x0401
#define EYE_CONTROL 0x0402
#define EXIT 9

#define QUICK_SAVE 0
#define DIALOG_SAVE 1
#define MAX_VIEWPORTS 4

// Extra definitions
#define DEBUG false


/*!
  The window ID as returned by glutCreateWindow()
*/
GLint windowID;

/*
  The width and height of the entire window. These values change when
  viewports are created or removed.
*/
GLint windowWidth; //!< Width of entire widow (changes when viewports are added or removed)
GLint windowHeight; //!< Height of entire window (chamges wen viewports are added or removed)

/*
  The x and y values of the mouse. These values are taken when a user
  presses the left mouse button.
*/
GLint mouseX; //!< The x value of the mouse, taken when user presses the left mouse button
GLint mouseY; //!< The y value of the mouse, taken when user presses the left mouse button

/*!
  The viewports that render the scene. This controls the viewports of
  the window. There are many events that call member functions of this
  variable.
*/
ViewportController * viewports;

/*!
  The filename of the output file. This contains the value passed to
  the program with the -o option.
*/
std::string outputfile;

/*
  The menu ID's as returned by glutCreateMenu().
*/
GLint menuID; //!< GLUT main menu ID
GLint viewportMenuID; //!< GLUT viewport menu ID
GLint modelMenuID; //!< GLUT model menu ID
GLint controlMenuID; //!< GLUT control menu ID
GLint decimationMenuID; //!< GLUT decimation menu ID
GLint decPerMenuID; //!< GLUT decimation/percentage menu ID
GLint decDisMenuID; //!< GLUT decimation/distance menu ID

/*
  Settings for our light. Currently there are only three defined
  attributes. We may not need these at all.
*/
GLfloat Light_Ambient[4]; //!< Ambient Light in the Scene
GLfloat Light_Diffuse[4]; //!< Diffuse Light in the Scene
GLfloat Light_Position[4]; //!< Position of Light in the Scene

/*
  Default settings: makes it less of a pain from the command line
*/
bool percent; //!< Default percent to decimate to
bool counter; //!< Default setting from command line
bool gui; //!< Default setting from command line (gui mode)
